from __future__ import annotations

from copy import deepcopy as dp
from datetime import datetime

import requests

from .rawquery import get_details_from_cloudmusic, get_search_results_from_cloudmusic
from .structures import SearchResult, SongDetail


class CloudMusicSongDetail(SongDetail):
    def __init__(self, raw_result: dict[str, str | int | list | dict | None]) -> None:
        self._raw_result = dp(raw_result)

    @property
    def album(self) -> str | None:
        alb: dict[str, str | int | list[str]] | None = self.type_filter(self._raw_result.get('al'), dict)
        if alb:
            albname = self.type_filter(alb.get('name'), str)
            return albname

    @property
    def albumid(self) -> int | None:
        alb: dict[str, str | int | list[str]] | None = self.type_filter(self._raw_result.get('al'), dict)
        if alb:
            albid = self.type_filter(alb.get('id'), int)
            return albid

    @property
    def aliases(self) -> list[str]:
        alia: list[str] | None = self.type_filter(self._raw_result.get('alia'), list)
        ret: list[str] = []
        if alia:
            for item in alia:
                if item:
                    ret.append(self.type_filter(item, str))

        return ret

    @property
    def translations(self) -> list[str]:
        tns: list[str] | None = self.type_filter(self._raw_result.get('tns'), list)
        ret: list[str] = []
        if tns:
            for item in tns:
                if item:
                    ret.append(self.type_filter(item, str))

        return ret

    @property
    def artists(self) -> list[str]:
        arts: list[dict[str, str | int | list[str]]] | None = self.type_filter(self._raw_result.get('ar'), list)
        ret: list[str] = []
        if arts:
            for item in arts:
                artist_name = self.type_filter(item.get('name'), str)
                if artist_name:
                    ret.append(artist_name)

        return ret

    @property
    def artistids(self) -> list[int]:
        arts: list[dict[str, str | int | list[str]]] | None = self.type_filter(self._raw_result.get('ar'), list)
        ret: list[int] = []
        if arts:
            for item in arts:
                artist_id = self.type_filter(item.get('id'), int)
                if artist_id:
                    ret.append(artist_id)

        return ret

    @property
    def coverurl(self) -> str | None:
        alb: dict[str, str | int | list[str]] | None = self.type_filter(self._raw_result.get('al'), dict)
        if alb:
            pic_url = self.type_filter(alb.get('picUrl'), str)
            return pic_url

    @property
    def songname(self) -> str | None:
        return self.type_filter(self._raw_result.get('name'), str)

    @property
    def songid(self) -> int | None:
        return self.type_filter(self._raw_result.get('id'), int)

    @property
    def publish_time(self) -> datetime | None:
        time_us = self.type_filter(self._raw_result.get('publishTime'), int)
        if time_us is not None:
            time_ms: float = time_us / 1000
            return datetime.fromtimestamp(time_ms)

    @property
    def genre(self) -> list[str]:
        return []

    @property
    def company(self) -> list[str]:
        return []


class CloudMusicSearchResult(SearchResult):
    def __init__(self, raw_result: dict[str, str | int | list | dict]) -> None:
        self._raw_result = dp(raw_result)

    @property
    def album(self) -> str | None:
        alb: dict[str, str | int | list[str]] | None = self.type_filter(self._raw_result.get('al'), dict)
        if alb:
            albname = self.type_filter(alb.get('name'), str)
            return albname

    @property
    def albumid(self) -> int | None:
        alb: dict[str, str | int | list[str]] | None = self.type_filter(self._raw_result.get('al'), dict)
        if alb:
            albid = self.type_filter(alb.get('id'), int)
            return albid

    @property
    def aliases(self) -> list[str]:
        alia: list[str] | None = self.type_filter(self._raw_result.get('alia'), list)
        ret: list[str] = []
        if alia:
            for item in alia:
                if item:
                    ret.append(self.type_filter(item, str))

        return ret

    @property
    def translations(self) -> list[str]:
        tns: list[str] | None = self.type_filter(self._raw_result.get('tns'), list)
        ret: list[str] = []
        if tns:
            for item in tns:
                if item:
                    ret.append(self.type_filter(item, str))

        return ret

    @property
    def artists(self) -> list[str]:
        arts: list[dict[str, str | int | list[str]]] | None = self.type_filter(self._raw_result.get('ar'), list)
        ret: list[str] = []
        if arts:
            for item in arts:
                artist_name = self.type_filter(item.get('name'), str)
                if artist_name:
                    ret.append(artist_name)

        return ret

    @property
    def artistids(self) -> list[int]:
        arts: list[dict[str, str | int | list[str]]] | None = self.type_filter(self._raw_result.get('ar'), list)
        ret: list[int] = []
        if arts:
            for item in arts:
                artist_id = self.type_filter(item.get('id'), int)
                if artist_id:
                    ret.append(artist_id)

        return ret

    @property
    def coverurl(self) -> str | None:
        alb: dict[str, str | int | list[str]] | None = self.type_filter(self._raw_result.get('al'), dict)
        if alb:
            pic_url = self.type_filter(alb.get('picUrl'), str)
            return pic_url

    @property
    def songname(self) -> str | None:
        return self.type_filter(self._raw_result.get('name'), str)

    @property
    def songid(self) -> int | None:
        return self.type_filter(self._raw_result.get('id'), int)

    @property
    def publish_time(self) -> datetime | None:
        time_us = self.type_filter(self._raw_result.get('publishTime'), int)
        if time_us is not None:
            time_ms: float = time_us / 1000
            return datetime.fromtimestamp(time_ms)

    def get_detail(self) -> CloudMusicSongDetail | None:
        if self.songid is not None:
            ret = details(self.songid)
            if len(ret) != 0:
                return ret[0]


def search(*keywords: str,
           result_pageidx: int = 0,
           result_size: int = 10,
           ) -> list[CloudMusicSearchResult]:
    """根据关键词，从 QQ 音乐获取匹配关键词的歌曲的信息。

    Args:
        keywords (str): 关键词
        result_pageidx (int): 搜索结果的所在的页码，默认为 0
        result_size (int): 搜索结果的数量，默认为 10
    Raises:
        requests.RequestException: 网络、远端相关错误
    """
    _full_result = get_search_results_from_cloudmusic(*keywords, result_pageidx=result_pageidx, result_size=result_size)
    if isinstance(_full_result, requests.Response):
        full_result: dict = _full_result.json()
    else:
        full_result: dict = _full_result
    ret = []
    if full_result:
        raw_results: list[dict] = full_result['result']['songs']
        for item in raw_results:
            ret.append(CloudMusicSearchResult(item))

    return ret


def details(*songids: int | str) -> list[CloudMusicSongDetail]:
    """根据一个或多个 songid 从网易云音乐获取一首/多首歌曲的详细信息。

    Args:
        *songids: 一个或多个歌曲 ID
    """
    _full_result = get_details_from_cloudmusic(*songids, raw_response=False)
    if isinstance(_full_result, requests.Response):
        full_result: dict = _full_result.json()
    else:
        full_result: dict = _full_result
    ret = []
    if full_result:
        raw_results: list[dict] = full_result['songs']
        for item in raw_results:
            ret.append(CloudMusicSongDetail(item))

    return ret



from __future__ import annotations

from abc import abstractmethod
from datetime import datetime
from typing import Any, Generic, Type

from .utils import T, T_OUT, type_filter


class SearchResult(Generic[T]):
    @property
    @abstractmethod
    def album(self) -> str | None:
        pass

    @property
    @abstractmethod
    def albumid(self) -> int | None:
        pass

    @property
    @abstractmethod
    def aliases(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def translations(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def artists(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def artistids(self) -> list[int]:
        pass

    @property
    @abstractmethod
    def coverurl(self) -> str | None:
        pass

    @property
    @abstractmethod
    def songname(self) -> str | None:
        pass

    @property
    @abstractmethod
    def songid(self) -> int | None:
        pass

    @property
    @abstractmethod
    def publish_time(self) -> datetime | None:
        pass

    @property
    def property_sep(self) -> str:
        return '、'

    def __repr__(self) -> str:
        ret_strseg = ['<']

        if self.songname:
            ret_strseg.append(f'name: {self.songname}')
        else:
            ret_strseg.append(f'noname')
        if self.translations:
            ret_strseg.append(f'trans: {self.property_sep.join(self.translations)}')
        if self.aliases:
            ret_strseg.append(f'aliases: {self.property_sep.join(self.aliases)}')
        if self.artists:
            ret_strseg.append(f'artists: {self.property_sep.join(self.artists)}')
        if self.album:
            ret_strseg.append(f'album: {self.album}')
        if self.publish_time:
            ret_strseg.append(f'publish time: {self.publish_time}')

        if len(ret_strseg) <= 1:
            ret_strseg.append('empty result')

        return '\n    '.join(ret_strseg) + '\n>'

    @classmethod
    def type_filter(cls, value: Any, t: Type[T_OUT], allow_None=True) -> T_OUT:
        return type_filter(value=value, t=t, allow_None=allow_None)

    def get_detail(self) -> SongDetail | None:
        pass


class SongDetail(SearchResult):
    @property
    @abstractmethod
    def album(self) -> str | None:
        pass

    @property
    @abstractmethod
    def albumid(self) -> int | None:
        pass

    @property
    @abstractmethod
    def aliases(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def translations(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def artists(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def artistids(self) -> list[int]:
        pass

    @property
    @abstractmethod
    def coverurl(self) -> str | None:
        pass

    @property
    @abstractmethod
    def songname(self) -> str | None:
        pass

    @property
    @abstractmethod
    def songid(self) -> int | None:
        pass

    @property
    @abstractmethod
    def publish_time(self) -> datetime | None:
        pass

    @property
    @abstractmethod
    def genre(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def company(self) -> list[str]:
        pass

    def __repr__(self) -> str:
        ret_strseg = ['<']

        if self.songname:
            ret_strseg.append(f'name: {self.songname}')
        else:
            ret_strseg.append(f'noname')
        if self.translations:
            ret_strseg.append(f'trans: {self.property_sep.join(self.translations)}')
        if self.aliases:
            ret_strseg.append(f'aliases: {self.property_sep.join(self.aliases)}')
        if self.artists:
            ret_strseg.append(f'artists: {self.property_sep.join(self.artists)}')
        if self.album:
            ret_strseg.append(f'album: {self.album}')
        if self.publish_time:
            ret_strseg.append(f'publish time: {self.publish_time}')
        if self.genre:
            ret_strseg.append(f'genre: {self.property_sep.join(self.genre)}')
        if self.company:
            ret_strseg.append(f'company: {self.property_sep.join(self.company)}')

        if len(ret_strseg) <= 1:
            ret_strseg.append('empty result')

        return '\n    '.join(ret_strseg) + '\n>'



from __future__ import annotations

from typing import Any, Callable, List, Union

from . import cloudmusic
from . import qqmusic
from .structures import SearchResult

__VERSION__ = '0.1.0'

SearchFunc = Union[
    Callable[[Any], List[cloudmusic.CloudMusicSearchResult]],
    Callable[[Any], List[qqmusic.QQMusicSearchResult]]
]


def supported_sources() -> dict[str, SearchFunc]:
    """列出支持的歌曲信息搜索来源，以及对应的搜索入口函数。

    目前支持的搜索来源：

    - cloudmusic - 网易云音乐
    - qqmusic - QQ 音乐
    """
    return {
        'cloudmusic': cloudmusic.search,
        'qqmusic': qqmusic.search
    }



from __future__ import annotations

from copy import deepcopy as dp
from datetime import datetime

import requests

from .rawquery import get_details_from_qqmusic, get_search_results_from_qqmusic
from .structures import SearchResult, SongDetail
from .utils import type_filter


class QQMusicSongDetail(SongDetail):
    def __init__(self, raw_result: dict[str, str | int | list | dict | None]) -> None:
        raw_result = dp(raw_result)
        raw_result_data = self.type_filter(raw_result['data'], dict, allow_None=False)
        self._track_info: dict[str, str | int | list | dict] | None = raw_result_data.get('track_info')
        self._extra_info: dict[str, str] | None = raw_result_data.get('extras')
        self._misc_info: dict[str, dict[str, str | list]] | None = raw_result_data.get('info')

    @property
    def album(self) -> str | None:
        if self._track_info:
            alb: dict[str, str | int] | None = self.type_filter(self._track_info.get('album'), dict)
            if alb:
                return self.type_filter(alb.get('name'), str)

    @property
    def albumid(self) -> int | None:
        if self._track_info:
            alb: dict[str, str | int] | None = self.type_filter(self._track_info.get('album'), dict)
            if alb:
                return self.type_filter(alb.get('id'), int)

    @property
    def albummid(self) -> str | None:
        if self._track_info:
            alb: dict[str, str | int] | None = self.type_filter(self._track_info.get('album'), dict)
            if alb:
                return self.type_filter(alb.get('mid'), str)

    @property
    def aliases(self) -> list[str]:
        ret = []
        if self._track_info:
            subtitle: str | None = self.type_filter(self._track_info.get('subtitle'), str)
            if subtitle:
                ret.append(subtitle)

        return ret

    @property
    def translations(self) -> list[str]:
        ret = []
        if self._extra_info:
            transname: str | None = self.type_filter(self._extra_info.get('transname'), str)
            if transname:
                ret.append(transname)

        return ret

    @property
    def artists(self) -> list[str]:
        ret = []
        if self._track_info:
            singers: list[dict[str, str | int]] | None = self.type_filter(self._track_info.get('singer'), list)
            if singers:
                for item in singers:
                    item = self.type_filter(item, dict)
                    singername: str | None = self.type_filter(item.get('name'), str)
                    if singername:
                        ret.append(singername)

        return ret

    @property
    def artistids(self) -> list[int]:
        ret = []
        if self._track_info:
            singers: list[dict[str, str | int]] | None = self.type_filter(self._track_info.get('singer'), list)
            if singers:
                for item in singers:
                    item = self.type_filter(item, dict)
                    singerid: int | None = self.type_filter(item.get('id'), int)
                    if singerid:
                        ret.append(singerid)

        return ret

    @property
    def artistmids(self) -> list[str]:
        ret = []
        if self._track_info:
            singers: list[dict[str, str | int]] | None = self.type_filter(self._track_info.get('singer'), list)
            if singers:
                for item in singers:
                    item = self.type_filter(item, dict)
                    singermid: str | None = self.type_filter(item.get('mid'), str)
                    if singermid:
                        ret.append(singermid)

        return ret

    @property
    def coverurl(self) -> str | None:
        albummid = self.type_filter(self.albummid, str)
        if albummid:
            return f'https://y.qq.com/music/photo_new/T002R800x800M000{albummid}.jpg'

    @property
    def songname(self) -> str | None:
        if self._track_info:
            return self.type_filter(self._track_info.get('name'), str)

    @property
    def songid(self) -> int | None:
        if self._track_info:
            return self.type_filter(self._track_info.get('id'), int)

    @property
    def songmid(self) -> str | None:
        if self._track_info:
            return self.type_filter(self._track_info.get('mid'), str)

    @property
    def publish_time(self) -> datetime | None:
        if self._track_info:
            pub_time_str = self.type_filter(self._track_info.get('time_public'), str)
            if pub_time_str:
                return datetime.fromisoformat(pub_time_str)

    @property
    def genre(self) -> list[str]:
        ret = []
        if self._misc_info:
            genre_info: dict[str, str | list[dict]] | None = self.type_filter(self._misc_info.get('genre'), dict)
            if genre_info:
                genre_contents: list[dict[str, str | int]] | None = self.type_filter(genre_info.get('content'), list)
                if genre_contents:
                    for item in genre_contents:
                        item = self.type_filter(item, dict)
                        genre_value: str | None = self.type_filter(item.get('value'), str)
                        if genre_value:
                            ret.append(genre_value)

        return ret

    @property
    def company(self) -> list[str]:
        ret = []
        if self._misc_info:
            company_info: dict[str, str | list[dict]] | None = self.type_filter(self._misc_info.get('company'), dict)
            if company_info:
                company_contents: list[dict[str, str | int]] | None = self.type_filter(company_info.get('content'), list)
                if company_contents:
                    for item in company_contents:
                        item = self.type_filter(item, dict)
                        company_value: str | None = self.type_filter(item.get('value'), str)
                        if company_value and company_value != '制作家':
                            ret.append(company_value)

        return ret


class QQMusicSearchResult(SearchResult):
    def __init__(self, raw_result: dict[str, str | int | list | dict]) -> None:
        self._raw_result = dp(raw_result)

    @property
    def album(self) -> str | None:
        return self.type_filter(self._raw_result.get('albumname'), str)

    @property
    def albumid(self) -> int | None:
        return self.type_filter(self._raw_result.get('albumid'), int)

    @property
    def albummid(self) -> str | None:
        return self.type_filter(self._raw_result.get('albummid'), str)

    @property
    def aliases(self) -> list[str]:
        lyric_title = self.type_filter(self._raw_result.get('lyric'), str)
        ret: list[str] = []
        if lyric_title:
            ret.extend(lyric_title.split('|'))

        return ret

    @property
    def translations(self) -> list[str]:
        return []

    @property
    def artists(self) -> list[str]:
        singers: list[dict[str, str | int]] | None = self.type_filter(self._raw_result.get('singer'), list)
        ret: list[str] = []
        if singers:
            for item in singers:
                singername = self.type_filter(item.get('name'), str)
                if singername:
                    ret.append(singername)

        return ret

    @property
    def artistids(self) -> list[int]:
        singers: list[dict[str, str | int]] | None = self.type_filter(self._raw_result.get('singer'), list)
        ret: list[int] = []
        if singers:
            for item in singers:
                singerid = self.type_filter(item.get('id'), int)
                if singerid:
                    ret.append(singerid)

        return ret

    @property
    def artistmids(self) -> list[str]:
        singers: list[dict[str, str | int]] | None = self.type_filter(self._raw_result.get('singer'), list)
        ret: list[str] = []
        if singers:
            for item in singers:
                singermid = self.type_filter(item.get('mid'), str)
                if singermid:
                    ret.append(singermid)

        return ret

    @property
    def coverurl(self) -> str | None:
        albummid = self.type_filter(self.albummid, str)
        if albummid:
            return f'https://y.qq.com/music/photo_new/T002R800x800M000{albummid}.jpg'

    @property
    def songname(self) -> str | None:
        return self.type_filter(self._raw_result.get('songname'), str)

    @property
    def songid(self) -> int | None:
        return self.type_filter(self._raw_result.get('songid'), int)

    @property
    def songmid(self) -> str | None:
        return self.type_filter(self._raw_result.get('songmid'), str)

    @property
    def publish_time(self) -> datetime | None:
        time_ms = self.type_filter(self._raw_result.get('pubtime'), int)
        if time_ms:
            return datetime.fromtimestamp(time_ms)

    def get_detail(self) -> SongDetail | None:
        if self.songmid:
            ret = details(self.songmid)
            if len(ret) != 0:
                return ret[0]


def search(*keywords: str,
           result_pageidx: int = 0,
           result_size: int = 10,
           ) -> list[QQMusicSearchResult]:
    """根据关键词，从 QQ 音乐获取匹配关键词的歌曲的信息。

    Args:
        keywords (str): 关键词
        result_pageidx (int): 搜索结果的所在的页码，默认为 0
        result_size (int): 搜索结果的数量，默认为 10
    Raises:
        requests.RequestException: 网络、远端相关错误
    """
    _full_result = get_search_results_from_qqmusic(*keywords, result_pageidx=result_pageidx, result_size=result_size)
    if isinstance(_full_result, requests.Response):
        full_result: dict = _full_result.json()
    else:
        full_result: dict = _full_result
    ret = []
    if full_result:
        raw_results: list[dict] = full_result['data']['song']['list']
        for item in raw_results:
            ret.append(QQMusicSearchResult(item))

    return ret


def details(*songmids: str) -> list[QQMusicSongDetail]:
    """根据一个或多个 songmid 从 QQ 音乐获取一首/多首歌曲的详细信息。

    Args:
        *songmids: 一个或多个歌曲 mID
    """
    _full_result = get_details_from_qqmusic(*songmids, raw_response=False)
    if isinstance(_full_result, requests.Response):
        full_result: dict = _full_result.json()
    else:
        full_result: dict = type_filter(_full_result, dict)
    ret = []
    if full_result:
        raw_results: list[dict] = full_result['songs']
        for item in raw_results:
            ret.append(QQMusicSongDetail(item['songinfo']))

    return ret



from __future__ import annotations

import json

import requests


def get_search_results_from_qqmusic(*keywords: str,
                                    result_pageidx: int = 0,
                                    result_size: int = 10,
                                    result_type: int = 0,
                                    raw_response=False
                                    ) -> dict | requests.Response:
    """从 QQ 音乐获取 歌曲/歌单/歌词/专辑/歌手/MV 的信息。

    Args:
        keywords (str): 关键词
        result_pageidx (int): 搜索结果的所在的页码，默认为 0
        result_size (int): 搜索结果的数量，默认为 10
        result_type (int): 搜索的类型
        raw_response (bool): 返回原始响应对象；
            默认为否（返回一个对结果进行 JSON 反序列化后得到的 dict）
    Raises:
        ValueError: 为参数 ``result_type`` 指定了不支持的值
        requests.RequestException: 网络、远端相关错误

    参数 ``result_type`` 为搜索的类型，支持以下值：
        - 0：歌曲
        - 2：歌单
        - 7：歌词
        - 8：专辑
        - 9：歌手
        - 12：MV
    """
    if result_type not in (0, 2, 7, 8, 9, 12):
        raise ValueError(f'不支持的搜索类型：{repr(result_type)}')

    final_keyword: str = ' '.join(keywords)

    if result_type == 2:
        url = 'https://c.y.qq.com/soso/fcgi-bin/client_music_search_songlist'
        params = {
            'remoteplace': 'txt.yqq.playlist',
            'page_no': result_pageidx,
            'num_per_page': result_size,
            'query': final_keyword
        }
    else:
        url = 'http://c.y.qq.com/soso/fcgi-bin/client_search_cp'
        params = {
            'format': 'json',
            'n': result_size,
            'p': result_pageidx + 1,
            'w': final_keyword,
            'cr': 1,
            'g_tk': 5381,
            't': result_type
        }
    headers = {
        'Referer': 'https://y.qq.com'
    }

    resp = requests.get(url=url, headers=headers, params=params)
    resp.raise_for_status()
    if raw_response:
        return resp
    return resp.json()


def get_search_results_from_cloudmusic(*keywords: str,
                                       result_pageidx: int = 0,
                                       result_size: int = 10,
                                       result_type: int = 1,
                                       raw_response=False
                                       ) -> dict | requests.Response:
    """从网易云音乐获取 歌曲/专辑/歌手/歌单/用户/MV/歌词/电台 的信息。

    Args:
        keywords (str): 关键词
        result_pageidx (int): 搜索结果的所在的页码，默认为 0
        result_size (int): 搜索结果的数量，默认为 10
        result_type (int): 搜索的类型
        raw_response (bool): 返回原始响应对象；
            默认为否（返回一个对结果进行 JSON 反序列化后得到的 dict）
    Raises:
        ValueError: 为参数 ``result_type`` 指定了不支持的值
        requests.RequestException: 网络、远端相关错误

    参数 ``result_type`` 为搜索的类型，支持以下值：
        - 1：歌曲
        - 10：专辑
        - 100：歌手
        - 1000：歌单
        - 1002：用户
        - 1004：MV
        - 1006：歌词
        - 1009：电台
    """
    if result_type not in (1, 10, 100, 1000, 1002, 1004, 1006, 1009):
        raise ValueError(f'不支持的搜索类型：{repr(result_type)}')

    final_keyword: str = ' '.join(keywords)

    url = 'https://music.163.com/api/cloudsearch/pc'
    payload = {
        's': final_keyword,
        'type': result_type,
        'limit': result_size,
        'offset': result_pageidx,
        'total': True
    }

    resp = requests.post(url=url, data=payload)
    resp.raise_for_status()
    if raw_response:
        return resp
    return resp.json()


def get_details_from_qqmusic(*songmids: str,
                             raw_response=False
                             ) -> dict[str, list[dict]] | list[requests.Response]:
    """从 QQ 音乐获取一首/多首歌曲的详细信息。

    QQ 音乐的查询接口只允许同时查询一首歌曲的信息，
    因此本函数的本质是对此接口的重复调用。

    Args:
        *songmids: 一个或多个歌曲 ID
        raw_response (bool): 返回一个或多个原始响应对象；
            默认为否（返回一个或多个对结果进行 JSON 反序列化后得到的 dict）
    """
    url = 'http://u.y.qq.com/cgi-bin/musicu.fcg'

    def simple_query(songmid: str) -> requests.Response:
        params = {
            'data': json.dumps(
                {
                    'songinfo': {
                        'method': 'get_song_detail_yqq',
                        'module': 'music.pf_song_detail_svr',
                        'param': {
                            'song_mid': songmid
                        }
                    }
                }
            )
        }

        resp = requests.get(url=url, params=params)
        resp.raise_for_status()
        return resp

    if raw_response:
        return [simple_query(_) for _ in songmids]
    return {'songs': [simple_query(_).json() for _ in songmids]}


def get_details_from_cloudmusic(*songids: int | str,
                                raw_response=False
                                ) -> dict | requests.Response:
    """从网易云音乐获取一首/多首歌曲的详细信息。

    Args:
        *songids: 一个或多个歌曲 ID
        raw_response (bool): 返回原始响应对象；
            默认为否（返回一个对结果进行 JSON 反序列化后得到的 dict）
    """
    songids_ = [int(_) for _ in songids]
    payload = {
        'c': json.dumps(
            [{'id': _} for _ in songids_]
        )
    }
    url = 'https://music.163.com/api/v3/song/detail'

    resp = requests.post(url=url, data=payload)
    if raw_response:
        return resp
    return resp.json()



from __future__ import annotations

from typing import Any, Type, TypeVar

T = TypeVar('T')
T_OUT = TypeVar('T_OUT')


def type_filter(value: Any, t: Type[T_OUT], allow_None=True) -> T_OUT:
    if value is None and allow_None:
        return value
    if isinstance(value, t):
        return value
    raise ValueError(f"unexpected type of value from result "
                     f"(should be '{t.__name__}', got '{type(value).__name__}')"
                     )
